/*
 *  ALTerator - ALT Linux configuration project
 *
 *  Copyright (c) 2003-2005 ALT Linux Ltd.
 *  Copyright (c) 2003-2005 Alexey Voinov
 *  Copyright (c) 2004,2005 Stanislav Ievlev
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
 *  USA.
 */
#include <utils/os.hh>
#include <utils/str.hh>
#include <utils/err.hh>

#include <sys/wait.h>

void
alt::run_shell(const std::string& shell, const std::string& input)
{
	pid_t child;
	int p[2];
	::pipe(p);
	if(!(child = fork()))
	{
		close(p[1]);
		close(0); dup2(p[0], 0);
		execl(shell.c_str(), shell.c_str(), NULL);
		exit(1);
	}
	else
	{
		close(p[0]);
		size_t pos = 0;
		while(pos < input.size())
		{
			ssize_t res = write(p[1],
					input.data() + pos, input.size() - pos);
			if(res < 0) throw err::general(
						"cannot send script to shell");
			pos += res;
		}
		close(p[1]);
		int status;
		waitpid(child, &status, 0);
		if(WEXITSTATUS(status) != 0)
			throw err::general("shell returned error");
	}

}

void
alt::exec(const std::string& cmd)
{
	args_iterator i(cmd);
	alt::exec(*i, i, args_iterator());
}

alt::proc_stat::proc_stat(pid_t pid_)
	: pid(pid_), exited(false), signaled(false), stopped(false)
{
}

alt::proc_stat::proc_stat(pid_t pid_, int status_)
	: pid(pid_),
	  exited(WIFEXITED(status_)),
	  signaled(WIFSIGNALED(status_)),
	  stopped(WIFSTOPPED(status_))
{
	if(exited) status = WEXITSTATUS(status);
	if(signaled) signal = WTERMSIG(status);
	if(stopped) signal = WSTOPSIG(status);
}

alt::proc_stat alt::wait(void)
{
	int status;
	pid_t pid = ::wait(&status);
	if(pid == -1) throw err::sys();
	return proc_stat(pid, status);
}

alt::proc_stat alt::wait(pid_t pid, wait_hang opt1, wait_traced opt2)
{
	int status = 0;
	pid = ::waitpid(pid, &status,
			(opt1 ? WNOHANG : 0) | (opt2 ? WUNTRACED : 0));
	if(pid == -1) throw err::sys();
	if(pid != 0) return proc_stat(pid, status);
	return proc_stat(pid);
}
